namespace Syntriax.Engine.Core;

/// <summary>
/// Represents a range of values along a single axis.
/// </summary>
/// <param name="min">The minimum value of the projection.</param>
/// <param name="max">The maximum value of the projection.</param>
/// <remarks>
/// Initializes a new instance of the <see cref="Projection1D"/> struct with the specified minimum and maximum values.
/// </remarks>
[System.Diagnostics.DebuggerDisplay("Min: {Min}, Max: {Max}")]
public readonly struct Projection1D(float min, float max)
{
    /// <summary>
    /// Gets the minimum value of the projection.
    /// </summary>
    public readonly float Min = min;

    /// <summary>
    /// Gets the maximum value of the projection.
    /// </summary>
    public readonly float Max = max;

    /// <summary>
    /// Checks if two projections overlap.
    /// </summary>
    /// <param name="left">The first projection to check.</param>
    /// <param name="right">The second projection to check.</param>
    /// <returns><see cref="true"/> if the projections overlap; otherwise, <see cref="false"/>.</returns>
    public static bool Overlaps(Projection1D left, Projection1D right) => Overlaps(left, right, out float _);

    /// <summary>
    /// Checks if two projections overlap and calculates the depth of the overlap.
    /// </summary>
    /// <param name="left">The first projection to check.</param>
    /// <param name="right">The second projection to check.</param>
    /// <param name="depth">The depth of the overlap, if any.</param>
    /// <returns><see cref="true"/> if the projections overlap; otherwise, <see cref="false"/>.</returns>
    public static bool Overlaps(Projection1D left, Projection1D right, out float depth)
    {
        // TODO Try to improve this
        bool rightMinInLeft = right.Min > left.Min && right.Min < left.Max;
        if (rightMinInLeft)
        {
            depth = left.Max - right.Min;
            return true;
        }

        bool rightMaxInLeft = right.Max < left.Max && right.Max > left.Min;
        if (rightMaxInLeft)
        {
            depth = left.Min - right.Max;
            return true;
        }

        bool leftMinInRight = left.Min > right.Min && left.Min < right.Max;
        if (leftMinInRight)
        {
            depth = right.Max - left.Min;
            return true;
        }

        bool leftMaxInRight = left.Max < right.Max && left.Max > right.Min;
        if (leftMaxInRight)
        {
            depth = right.Min - left.Max;
            return true;
        }

        depth = 0f;
        return false;
    }
}

/// <summary>
/// Provides extension methods for the <see cref="Projection1D"/> struct.
/// </summary>
public static class Projection1DExtensions
{
    /// <summary>
    /// Checks if two projections overlap.
    /// </summary>
    /// <param name="left">The first projection to check.</param>
    /// <param name="right">The second projection to check.</param>
    /// <returns><see cref="true"/> if the projections overlap; otherwise, <see cref="false"/>.</returns>
    public static bool Overlaps(this Projection1D left, Projection1D right) => Projection1D.Overlaps(left, right);

    /// <summary>
    /// Checks if two projections overlap and calculates the depth of the overlap.
    /// </summary>
    /// <param name="left">The first projection to check.</param>
    /// <param name="right">The second projection to check.</param>
    /// <param name="depth">The depth of the overlap, if any.</param>
    /// <returns><see cref="true"/> if the projections overlap; otherwise, <see cref="false"/>.</returns>
    public static bool Overlaps(this Projection1D left, Projection1D right, out float depth) => Projection1D.Overlaps(left, right, out depth);
}
